@@ -398,8 +398,7 @@ ebbm_connect_user (EBookBackendMAPI *ebma,
 			g_object_unref (old_conn);
 
 		if (!priv->conn || mapi_error) {
-			gboolean is_network_error = g_error_matches (mapi_error, E_MAPI_ERROR, MAPI_E_NETWORK_ERROR) ||
-				(mapi_error && mapi_error->domain != E_MAPI_ERROR);
+			gboolean is_network_error = mapi_error && mapi_error->domain != E_MAPI_ERROR;
 
 			if (priv->conn) {
 				g_object_unref (priv->conn);
@@ -1595,6 +1594,8 @@ e_book_backend_mapi_notify_contact_update (EBookBackendMAPI *ebma,
 	   it can happen when cache is filling and the book view started later */
 	if (!book_view)
 		book_view = ebbm_pick_book_view (ebma);
+	else
+		g_object_ref (book_view);
 
 	if (book_view) {
 		if (!e_book_backend_mapi_book_view_is_running (ebma, book_view))
