@@ -69,11 +69,55 @@ mapi_backend_get_settings (EMapiBackend *backend)
 	return CAMEL_MAPI_SETTINGS (settings);
 }
 
+static ESourceAuthenticationResult
+mapi_backend_try_password_sync (ESourceAuthenticator *authenticator,
+				const GString *password,
+				GCancellable *cancellable,
+				GError **error);
+
+static gpointer
+mapi_backend_authenticate_kerberos_thread (gpointer user_data)
+{
+	EMapiBackend *mapi_backend = user_data;
+	CamelMapiSettings *mapi_settings;
+	GError *error = NULL;
+
+	g_return_val_if_fail (E_IS_MAPI_BACKEND (mapi_backend), NULL);
+
+	mapi_settings = mapi_backend_get_settings (mapi_backend);
+	e_mapi_util_trigger_krb_auth_from_settings (mapi_settings, &error);
+
+	if (error) {
+		g_warning ("[evolution-mapi] Failed to trigger KrbAuthDialog: %s", error->message);
+		g_clear_error (&error);
+	}
+
+	mapi_backend_try_password_sync (E_SOURCE_AUTHENTICATOR (mapi_backend), NULL, NULL, NULL);
+
+	g_object_unref (mapi_backend);
+
+	return NULL;
+}
+
 static void
 mapi_backend_queue_auth_session (EMapiBackend *backend)
 {
+	CamelMapiSettings *mapi_settings;
+
+	mapi_settings = mapi_backend_get_settings (backend);
+
 	backend->priv->need_update_folders = FALSE;
 
+	/* kerberos doesn't use passwords, do it directly */
+	if (camel_mapi_settings_get_kerberos (mapi_settings)) {
+		GThread *thread;
+
+		thread = g_thread_new (NULL, mapi_backend_authenticate_kerberos_thread, g_object_ref (backend));
+		g_thread_unref (thread);
+
+		return;
+	}
+
 	/* For now at least, we don't need to know the
 	 * results, so no callback function is needed. */
 	e_backend_authenticate (
@@ -82,35 +126,6 @@ mapi_backend_queue_auth_session (EMapiBackend *backend)
 		NULL, NULL, NULL);
 }
 
-static void
-add_remote_sources (EMapiBackend *backend)
-{
-	GList *old_sources, *iter;
-	ESourceRegistryServer *registry;
-
-	registry = e_collection_backend_ref_server (E_COLLECTION_BACKEND (backend));
-	old_sources = e_collection_backend_claim_all_resources (E_COLLECTION_BACKEND (backend));
-	for (iter = old_sources; iter; iter = iter->next) {
-		ESource *source = iter->data;
-		ESourceMapiFolder *extension;
-		const gchar *foreign_username;
-
-		if (!e_source_has_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER))
-			continue;
-
-		/* foreign folders are just added */
-		extension = e_source_get_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER);
-		foreign_username = e_source_mapi_folder_get_foreign_username (extension);
-		if (e_source_mapi_folder_is_public (extension) || (foreign_username && *foreign_username)) {
-			e_server_side_source_set_writable (E_SERVER_SIDE_SOURCE (source), TRUE);
-			e_server_side_source_set_remote_deletable (E_SERVER_SIDE_SOURCE (source), TRUE);
-			e_source_registry_server_add_source (registry, source);
-		}
-	}
-	g_list_free_full (old_sources, g_object_unref);
-	g_object_unref (registry);
-}
-
 struct SyndFoldersData
 {
 	EMapiBackend *backend;
@@ -256,6 +271,37 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 		e_source_registry_server_remove_source (server, source);
 	}
 
+	all_sources = e_collection_backend_claim_all_resources (E_COLLECTION_BACKEND (backend));
+	for (citer = all_sources; citer; citer = citer->next) {
+		ESource *source = citer->data;
+		ESourceMapiFolder *extension;
+		const gchar *foreign_username;
+
+		if (!e_source_has_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER))
+			continue;
+
+		/* foreign folders are just added */
+		extension = e_source_get_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER);
+		foreign_username = e_source_mapi_folder_get_foreign_username (extension);
+		if (e_source_mapi_folder_is_public (extension) || (foreign_username && *foreign_username)) {
+			e_server_side_source_set_writable (E_SERVER_SIDE_SOURCE (source), TRUE);
+			e_server_side_source_set_remote_deletable (E_SERVER_SIDE_SOURCE (source), TRUE);
+			e_source_registry_server_add_source (server, source);
+		} else if (!has_gal && e_source_has_extension (source, E_SOURCE_EXTENSION_ADDRESS_BOOK)) {
+			ESourceAddressBook *book_ext;
+
+			book_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_ADDRESS_BOOK);
+			has_gal = g_strcmp0 ("mapigal", e_source_backend_get_backend_name (E_SOURCE_BACKEND (book_ext))) == 0;
+			if (has_gal)
+				e_source_registry_server_add_source (server, source);
+			else
+				e_source_remove_sync (source, NULL, NULL);
+		} else {
+			e_source_remove_sync (source, NULL, NULL);
+		}
+	}
+	g_list_free_full (all_sources, g_object_unref);
+
 	/* add GAL, if not there already */
 	if (!has_gal) {
 		ESource *source;
@@ -292,8 +338,6 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 		g_object_unref (source);
 	}
 
-	add_remote_sources (backend);
-
 	g_list_free_full (configured, g_object_unref);
 	g_object_unref (server);
 
